using System;
using System.IO;			// for StreamWriter I/O to a file
using System.Text;			// for Encoding I/O to a Text file
using System.Windows.Forms;	// for RichTextBox

namespace MyNamespace
{
	/// <summary>
	///		Common routines for FlyAKite program
	///		Developed in TURBO Pascal 5.5 and 6.0, converted to Visual C# .Net
	///
	/// ---------------- *** LEGAL NOTICE: *** ------------------
	///
	/// This program is free software; you can redistribute it and/or modify
	/// it under the terms of the GNU General Public License (GPL) as published
	/// by the Free Software Foundation. cf. the file COPYING.txt.
	///
	/// ------------ *** end of legal notice *** ------------------
	/// </summary>

	public class Common
	{
		public Common()		// constructor logic
		{
			dataFile = "Data.txt";
			logFile = "Log.txt";
			logg = false;
			nL = Environment.NewLine;
			nL2 = nL + nL;
			abortCalc = false;
			muAbort = false;
			quiet = false;
			muICount = 0;				// belongs to MuInterrupt()
			muITimeSaved = 0;			// belongs to MuInterrupt()
		}

		public static void SingleSpace() // guarantee a single line space in outS
		{
			SingleSpace(outS);
		}

		public static void SingleSpace(RichTextBox outB) // guarantee a single line space in output box
		{
			int iL = outB.Text.Length;
			if (iL > 0)
			{
				if (outB.Text.Substring(iL - 1) != "\n") WriteLn(outB, null);
			}
		}

		public static void DoubleSpace(RichTextBox outB) // guarantee a double line space in output box
		{
			int iL = outB.Text.Length;
			if (iL > 0)
			{
				if (outB.Text.Substring(iL - 1) != "\n") WriteLn(outB, null);
				if (outB.Text.Substring(outB.Text.Length - 2) != "\n\n") WriteLn(outB, null);
			}
		}

		public static void DoubleSpace() // guarantee a double line space in outS
		{
			DoubleSpace(outS);
		}

		public static bool OpenRead(out StreamReader inF, string fName, out string dir)
		// open file for Reading from CurrentDirectory or homeDir
		{
			string stEx1;
			string stEx2 = "";
			string savDir = Environment.CurrentDirectory.ToString();
			if (!quiet) DoubleSpace();
			dir = savDir;
			inF = null;
			if (OpenReadFile(out inF, fName, out stEx1))
			{
				if (!quiet)
				{
					WriteDLn("File " + fName + " opened for reading");
					WCurDir(fName);
				}
				return true;
			}
			if (savDir != homeDir)
			{
				dir = homeDir;
				Directory.SetCurrentDirectory(homeDir);
				if (OpenReadFile(out inF, fName, out stEx2))
				{
					if (!quiet)
					{
						WriteLn("File " + fName + " opened for reading");
						WCurDir(fName);
					}
					Directory.SetCurrentDirectory(savDir);
					return true;
				}
				Directory.SetCurrentDirectory(savDir);
			}
			dir = "";
			WriteLn("Cannot open file " + fName + " for input");
			WriteTLn("Not in current directory " + savDir);
			ReportError("OpenRead", stEx1);		// file not found most likely
			if (savDir != homeDir)
			{
				WriteTLn("Not in home directory " + homeDir);
				ReportError("OpenRead", stEx2); // file not found most likely
			}
			return false;
		} // OpenRead

		public static bool OpenReadFile(out StreamReader inF, string fName, out String stEx)
		// open file for Reading from CurrentDirectory
		{
			inF = null;
			stEx = "";
			try
			{
				inF = new StreamReader(fName); // open fName for input as inF
				return true;
			}
			catch (Exception ex)
			{
				stEx = ex.Message.ToString();
				return false;
			}
		}

		public static bool OpenWrite(out StreamWriter outF, string fName) // open file for Writing
		{
			outF = null; // outF must be assigned to before control leaves the current method
			try
			{
				// open fName for output as outF
				outF = new StreamWriter(fName, false, Encoding.ASCII); // Appending = false
				if (!quiet)
				{
					WriteDLn("File " + fName + " opened for writing");
					WCurDir(fName);
				}
				return true;
			}
			catch (Exception ex)
			{
				WriteDLn("Cannot open file " + fName + " for output");
				WCurDir(fName);
				ReportError("OpenWrite", ex.Message);
				return false;
			}
		}

		public static bool ChkWrite(out StreamWriter outF, string fName, string st)
		// check start write of file
		{
			if (!OpenWrite(out outF, fName))
			{
				if (MakeReady() == DialogResult.OK)
				{
					if (!OpenWrite(out outF, fName))
					{
						WriteLn("Aborting file write...");
						return false;
					}
				}
				else
				{
					WriteLn("Aborting file write...");
					return false;
				}
			}
			if (!quiet) WriteTLn("Writing file " + fName + st);
			return true;
		}

		public static bool OpenAppend(ref StreamWriter outF, string fName)	// open file for Writing
		{
			try
			{
				// open fName for Append as outF
				outF = new StreamWriter(fName, true, Encoding.ASCII);		//Appending = true
				return true;
			}
			catch (Exception ex)
			{
				WriteDLn("Cannot open file " + fName + " for output");
				WCurDir(fName);
				ReportError("OpenAppend", ex.Message);
				return false;
			}
		}

		public static bool ChkAppend(ref StreamWriter outF, string fName)
		// check start Append to a file
		{
			if (!OpenAppend(ref outF, fName))
			{
				if (MakeReady() == DialogResult.OK)
				{
					if (!OpenAppend(ref outF, fName))
					{
						WriteLn("Aborting file write...");
						return false;
					}
				}
			}
			return true;
		}

		public static void ReportError(string who, string errorMessage) // report IOResult etc. error, errorMessage = caught.Message
		{
			GC.Collect();	// force garbage collection of all generations
			if (errorMessage == "") return;
			WriteTLn(who + ": " + errorMessage);
		}

		public static DialogResult MakeReady() // pause for operator to make file/drive ready
		{
			DialogResult key = MessageBox.Show(Global.pName + ": Make file/drive ready and click \"OK\"\n" +
				StringOf(29, " ") + "or \"Cancel\"",
				Global.title,
				MessageBoxButtons.OKCancel,
				MessageBoxIcon.Exclamation);
			// pause ----------
			if (key != DialogResult.OK) WriteLn("I/O operation aborted by operator!");
			return key;
		}

		public static void ChangeDir(ref string st)
		// change the default disk directory and remove its name from command line
		{
			string stL;
			string savDir;
			if (st.Length == 0) st = "\""; // for safety
			stL = FileNm(ref st);
			savDir = "";
			try
			{
				if (!quiet) DoubleSpace();
				savDir = Environment.CurrentDirectory.ToString();
				if (stL == savDir) stL = "";
				if (stL != "") Directory.SetCurrentDirectory(stL);
			}
			catch
			{
				// -------- Trap for error handling
				WriteLn("Path not found!");
			}
			finally
			{
				if (!quiet)
				{
					if (Environment.CurrentDirectory.ToString() != savDir)
					{
						WriteTLn("Directory changed to " + Environment.CurrentDirectory.ToString());
					}
					else
					{
						WriteTLn("Directory not changed " + Environment.CurrentDirectory.ToString());
					}
					WriteTLn("Home directory is " + homeDir);
				}
			}
		}

		public static string FileNm(ref string st)
		// return and remove file name from string
		{
			int i;
			string stL;
			if (st.Length == 0)
			{
				return "";
			}
			i = st.IndexOf("\"", 1);
			if (i <= 0) i = st.Length;
			stL = st.Substring(1, i - 1);
			DelNPBls(ref st, i); // do not delete the trailing " if any
			return stL;
		}

		public static void ToggleLogg() //Toggle Logging of output screen to Log File
		{
			DateTimePicker dateTime = new System.Windows.Forms.DateTimePicker();
			string dateTimeString = dateTime.Value.ToString();
			if (!quiet) DoubleSpace();
			if (logg)
			{
				string st = "Log file " + logFile + " Closed " + dateTimeString;
				if (!quiet)
				{
					WriteLn(st);
					outL.WriteLine();
				}
				outL.Close();	// close the file
				logg = false;
			}
			else
			{
				if (logFile == "") logFile = "Log.txt";
				bool check;
				check = ChkAppend(ref outL, logFile);
				if (check) // open logFile for Append as outL
				{
					logg = true;
					string st = "Log file " + logFile + " Opened for Append " + dateTimeString;
					if (!quiet)
					{
						WriteDLn(st);
						WCurDir(logFile);
						outL.WriteLine(Global.title + ", " + Global.version);
						if (Global.system != "") outL.WriteLine("Run on: " + Global.system);
					}
				}
				else
				{
					return;
				}
			}
			Global.objRunForm.SetLogCaption(); // call sub in Run form
		} // ToggleLogg

		public static void ClearLog()	// clear Log file
		{
			DateTimePicker dateTime = new System.Windows.Forms.DateTimePicker();
			string dateTimeString = dateTime.Value.ToString();
			bool logS = logg;	// save Log status in a local
			if (!quiet) DoubleSpace();
			if (logg)
			{
				string st = "Log file " + logFile + " Closed " + dateTimeString;
				if (!quiet)
				{
					WriteLn(st);
					outL.WriteLine();
				}
				WriteLn(null, outL);
				outL.Close();		// close the file
				logg = false;
			}
			// open logFile for output as outL, clear the file, outL gotten by open routine
			if (ChkWrite(out outL, logFile, ""))
			{
				logg = true;		// set to get message to the file
				string st = "Log file " + logFile + " Cleared " + dateTimeString;
				if (!quiet)
				{
					WriteLn(st);
					WCurDir(logFile);
				}
				else
				{
					outL.WriteLine(st);
				}
				outL.WriteLine(Global.title + ", " + Global.version);
				if (Global.system != "") outL.WriteLine("Run on: " + Global.system);
				if (!quiet)
				{
					WriteLn();
				}
				else
				{
					outL.WriteLine();
				}
				if (!logS)
				{
					outL.Close();	// close the file
					logg = false;
				}
			}
		} // ClearLog

		public static void FixFName(ref string fName, string ext)
		// fix file name to path + name + .extension
		{
			fName = fName.Trim(' ');					// remove leading and trailing blanks
			int i = fName.IndexOf(" ");
			if (i >= 0) fName = fName.Substring(0, i);	// get rid if all blanks
			fName = fName.TrimEnd('.');					// remove trailing '.'
			if (fName == "") return;
			int p = fName.IndexOf(@"\");				// find last "\"
			i = fName.IndexOf('.', p+1);				// a "." after the last "\" starts extension
			if (i < 0 && ext != "")
				fName = fName + "." + ext;				// use default extension
			i = fName.IndexOf(":.");					// :. OR \. is a bad file name
			if (i < 0) i = fName.IndexOf(@"\.");
			if (i >= 0) fName = fName.Substring(0, i);	// if \.xxx, or :.xxx, .xxx deleted
		}

		public static void WCurDir(string fName)
		// write current disk directory with final \
		// using Environment.CurrentDirectory.ToString(): C:\ already has the \ but non-root directories do not
		{
			// this can garble some output, but good info
			string cD = Directory.GetCurrentDirectory();
			if (cD.Substring(cD.Length - 1) != @"\") cD += @"\";
			if (fName.Length > 1 && fName[1] == ':')
			{
				WriteTLn("Directory name = " + cD);
			}
			else
			{
				WriteTLn("Full name = " + cD + fName);
			}
		}

		public static void DelNPBls(ref string st, int n)
		// delete 1st n characters plus new leading blanks
		{
			if (n >= st.Length) { st = ""; return; }
			st = st.Substring(n).TrimStart(' ');
		}

		public static string StringOf(int n, string stIn) // repeat stIn, n times
		{
			string stOut = "";
			for (int i = 0; i < n; i++)
			{
				stOut += stIn;
			}
			return stOut;
		}

		public static void Write(string message) // write a short message to outS
		{
			Write(outS, null, message);
		}

		public static void WriteLn() // write a linefeed to outS
		{
			WriteLn(outS, null, "");
		}

		public static void WriteLn(string message) // write a short message and a linefeed to outS
		{
			WriteLn(outS, null, message);
		}

		public static void WriteLn(RichTextBox outB, StreamWriter outF) // Write linefeed
		{
			Write(outB, outF, nL);
		}

		public static void WriteLn(RichTextBox outB, StreamWriter outF, string st) // Write st plus linefeed
		{
			Write(outB, outF, st + nL);
		}

		public static void WriteLn2() // Write 2 linefeeds to outS
		{
			Write(outS, null, nL2);
		}

		public static void WriteLn2(string st) // Write st plus 2 linefeeds to outS
		{
			Write(outS, null, st + nL2);
		}

		public static void WriteLn2(RichTextBox outB, StreamWriter outF) // Write 2 linefeeds
		{
			Write(outB, outF, nL2);
		}

		public static void WriteLn2(RichTextBox outB, StreamWriter outF, string st) // Write st plus 2 linefeeds
		{
			Write(outB, outF, st + nL2);
		}

		public static void WriteSLn(string st) // Write SingleSpace, st, and linefeed to outS
		{
			SingleSpace(outS);
			Write(outS, null, st + nL);
		}

		public static void WriteDLn(string st) // Write DoubleSpace, st, and linefeed to outS
		{
			DoubleSpace(outS);
			Write(outS, null, st + nL);
		}

		public static void WriteDLn2(string st) // Write DoubleSpace, st, and 2 linefeeds to outS
		{
			DoubleSpace(outS);
			Write(outS, null, st + nL2);
		}

		public static void WriteDLn(RichTextBox outB, StreamWriter outF, string st) // Write DoubleSpace, st, and linefeed
		{
			DoubleSpace(outB);
			Write(outB, outF, st + nL);
		}

		public static void Write(RichTextBox outB, StreamWriter outF, string st) // print string to RichTextBox or to a file
		{
			if (st == "") return;
			if (outF != null)
			{
				try	{outF.Write(st);}
				catch (Exception ex) {ReportError("Write", ex.Message);}
			}
			if (outB != null)
			{
				outB.AppendText(st);
				if (st[(st.Length - 1)] == '\n')	// if a new line, move to bottom/end of output text box
				{
					Global.runTextOut.Select(Global.runTextOut.Text.Length, 0);
					Global.runTextOut.Focus();
				}
				if (logg && (outB == outS))			// only log output to Run form textbox
				{
					try	{outL.Write(st);}
					catch {}
				}
				if (outB.Text.Length > 200000)		// clear top of screen after about 50 full pages
				{
					int i = outB.Text.IndexOf("\n", 4000);
					outB.Text = outB.Text.Substring(i + 1);	// delete partial line
				}
			}
		} // Write

		public static void WriteT(string st)
		// WriteT with 0 tot and 0 width to outS
		{
			int tot = 0;
			SingleSpace();
			WriteT(outS, null, st, ref tot, 0);
		}

		public static void WriteT(string st, ref int tot)
		// WriteT with 0 width to outS
		{
			WriteT(outS, null, st, ref tot, 0);
		}

		public static void WriteT(RichTextBox outB, StreamWriter outF, string st)
		// WriteT with 0 tot and 0 width
		{
			int tot = 0;
			SingleSpace(outB);
			WriteT(outB, outF, st, ref tot, 0);
		}

		public static void WriteT(RichTextBox outB, StreamWriter outF, string st, ref int tot)
		// WriteT with 0 width
		{
			WriteT(outB, outF, st, ref tot, 0);
		}

		public static void WriteTLn()
		// WriteTLn with "", 0 tot and 0 width to outS
		{
			Write(outS, null, nL);
		}

		public static void WriteTLn(string st)
		// WriteTLn with 0 tot and 0 width to outS
		{
			int tot = 0;
			SingleSpace();
			if (st != "") WriteT(outS, null, st, ref tot, 0);
			Write(outS, null, nL);
		}

		public static void WriteTLn(string st, ref int tot)
		// WriteT and linefeed to outS
		{
			if (st != "") WriteT(outS, null, st, ref tot, 0);
			Write(outS, null, nL);
			tot = 0;
		}

		public static void WriteTLn(RichTextBox outB, StreamWriter outF, string st)
		// WriteT and linefeed with 0 tot and 0 width
		{
			int tot = 0;
			SingleSpace(outB);
			if (st != "") WriteT(outB, outF, st, ref tot, 0);
			Write(outB, outF, nL);
		}

		public static void WriteTLn(RichTextBox outB, StreamWriter outF, string st, ref int tot)
		// WriteT and linefeed with 0 width
		{
			if (st != "") WriteT(outB, outF, st, ref tot, 0);
			Write(outB, outF, nL);
			tot = 0;
		}

		public static void WriteTLn(RichTextBox outB, StreamWriter outF, string st, ref int tot, int width)
		// WriteT and linefeed
		{
			if (st != "") WriteT(outB, outF, st, ref tot, width);
			Write(outB, outF, nL);
			tot = 0;
		}

		public static void WriteTLn2(string st)
		// WriteTLn2 with 0 tot and 0 width to outS
		{
			int tot = 0;
			SingleSpace();
			if (st != "") WriteT(outS, null, st, ref tot, 0);
			Write(outS, null, nL2);
		}

		public static void WriteTLn2(RichTextBox outB, StreamWriter outF, string st)
		// WriteTLn2 with 0 tot and 0 width
		{
			int tot = 0;
			SingleSpace(outB);
			if (st != "") WriteT(outB, outF, st, ref tot, 0);
			Write(outB, outF, nL2);
		}

		public static void WriteTLn2(RichTextBox outB, StreamWriter outF, string st, ref int tot, int width)
		// WriteT and a double linefeed after
		{
			if (st != "") WriteT(outB, outF, st, ref tot, width);
			Write(outB, outF, nL2);
			tot = 0;
		}

		public static void WriteTD(RichTextBox outB, StreamWriter outF, string st)
		// force double space and WriteT with 0 tot and 0 width
		{
			int tot = 0;
			SingleSpace(outB);
			WriteT(outB, outF, st, ref tot, 0);
		}

		public static void WriteTDLn(string st)
		// WriteT with a guaranteed double linefeed before and single linefeed after to outS
		{
			DoubleSpace(outS);
			WriteTLn(outS, null, st);
		}

		public static void WriteTDLn(RichTextBox outB, StreamWriter outF, string st)
		// WriteT with a guaranteed double linefeed before and single linefeed after
		{
			DoubleSpace(outB);
			WriteTLn(outB, outF, st);
		}

		public static void WriteT(RichTextBox outB, StreamWriter outF, string st, ref int tot, int width)
		// output String and linefeed every 60, 80 or width characters
		// tot = characters on line so far
		{
			int i;
			int j;
			int iL;
			int n;
			string stE;
			if (width > 0)
				n = width;
			else
			{
				if (outB == null) n = 60;	// 80 for screen, 60 for files
				else n = 80;
			}
			i = 1;
			j = n - tot;
			iL = st.Length;
			while (iL > 0)
			{
				stE = "";
				if (tot >= n)
				{
					stE = nL;				// format n characters per line
					j = n;
					tot = 0;
				}
				if (iL <= j)
				{
					Write(outB, outF, stE + st.Substring(i-1));	// built for speed
					tot += iL;									// don't call Write too often
					iL = 0;
				}
				else
				{
					Write(outB, outF, stE + st.Substring(i-1, j));
					i += j;
					iL -= j;
					j = n;
					tot = n;
				}
			}
		} // WriteT

		public static void MuInterrupt() // test for ESC key pressed to abort operation
		{
			muICount++;
			if (muICount > 100)
			{
				muICount = 0;
				int Time = Environment.TickCount;	// time in milliseconds
				if (Time - muITimeSaved > 500 || Time < muITimeSaved)
				{
					// don't hog the processor, but don't slowdown too much
					Application.DoEvents();			// process user requested actions
					muITimeSaved = Time;
				}
			}
			RunForm.FocusBothUs();
		} // MuInterrupt
		
		public static bool abortCalc;	// true if abort calc button clicked
		public static bool configUp;	// true if Config form is up and running
		public static bool logg;		// true if Logging screen output to FILE
		public static bool muAbort;		// true if abort confirmed by operator
		public static bool quiet;		// true if "quiet" mode is on
		public static bool running;		// true if calculating
		public static Button runCmdLog; // Log command button on Run form
		public static int muICount;		// belongs to MuInterrupt()
		public static int muITimeSaved;	// belongs to MuInterrupt()
		public static int selectFilter;	// file selected FilterIndex the Config Form
		public static RichTextBox outS; // screen Output destination
		public static StreamWriter outL;// log file output #
		public static string dataFile;	// Data file name
		public static string homeDir;	// initial disk directory when program started
		public static string logFile;	// file name of log file
		public static string nL;		// one Environment.NewLine \n or \r or \r\n
		public static string nL2;		// nL + nL	}
		public static string selectFile;// file selected on the Config Form
		public static string selectPath;// path selected on the Config Form
	}
}