using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace MyNamespace
{
	/// <summary>
	/// Summary description for PlotControl.
	/// PlotControl is the User Control Component for testing the plotting functions in C#
	/// Developed by Michael Gold in 2002 and modified by Harry J. Smith in 2005
	/// See C# Corner at: http://www.c-sharpcorner.com/Code/2002/Aug/XYPlotControl.asp
	///
	/// ---------------- *** LEGAL NOTICE: *** ------------------
	///
	/// This program is free software; you can redistribute it and/or modify
	/// it under the terms of the GNU General Public License (GPL) as published
	/// by the Free Software Foundation.
	///
	/// ------------ *** end of legal notice *** ------------------
	/// </summary>
	public class PlotControl : System.Windows.Forms.UserControl
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		/// <summary>
		///	Required designer variable.
		/// </summary>

		private const int kXAxisIndent = 65;
		private const int kYAxisIndent = 41;
		public const int kScreenScale = 28;	// pixels per tick in both x and y
		public const int kPixelsForX = 925;	// pixels on x-axis approx.

		private GraphicsPath gpY = new GraphicsPath();
		private Font GraphFont = new Font("Arial", 8);
		private int m_nCurrentIndex = 0;
		private ArrayList m_points = new ArrayList();
		private ArrayList m_points2 = new ArrayList();
		private float m_ptXOrigin = 0.0f;
		private float m_ptYOrigin = 0.0f;
		private int m_nNumberOfPoints = 0;
		private float m_fXTickValue = 1.0f;
		private float m_fYTickValue = 1.0f;
		private string m_LabelX = "";
		private string m_LabelY = "";
		public bool m_bConnectPoints = false;
		private Pen ThePen = new Pen(Color.Blue);
		private Pen[] m_Pens = new Pen[5] {new Pen(Color.Red, 2), null, null, null, null};
		private int[] m_NextGraphStart = new int[5]{0,0,0,0,0};
		public int NumberOfPens = 1;

		public PlotControl()
		{
			// TODO: Add any initialization after the InitForm call
			// This call is required by the WinForms Form Designer.
			m_points.Add(new PointI());
			InitializeComponent();
		}

		/// <summary>
		///	Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		public Point[] GetGraphPoints(ArrayList points)
		{
			Point[] thePointsArray = new Point[points.Count];
			int i = 0;
			foreach (PointI pt in points)
			{
				thePointsArray[i] = pt.ToPoint();
				i++;
			}

			return thePointsArray;
		}

		protected override void OnPaint(PaintEventArgs pe)
		{
			Graphics g = pe.Graphics;
			Pen myPen = m_Pens[0];
			Rectangle rect = this.ClientRectangle; 
			DrawXAxis(g);
			DrawYAxis(g);
			int i;
			int j;
			if (m_nNumberOfPoints > 0)
			{
				if (m_bConnectPoints && m_nNumberOfPoints > 1)
				{
					for (int nextPen = 0; nextPen < NumberOfPens; nextPen++)
					{
						m_points2.Clear();
						int endPoint = (nextPen < (NumberOfPens - 1)) ? m_NextGraphStart[nextPen+1] : m_nNumberOfPoints;
						for (j = 0, i = m_NextGraphStart[nextPen]; i < endPoint; j++, i++)
						{
							m_points2.Add(m_points[i]);
						}
						try
						{
							g.DrawLines(m_Pens[nextPen], GetGraphPoints(m_points2));
						}
						catch
						{
							;
						}
					}
				}
				else
				{
					for (int nextPen = 0; nextPen < NumberOfPens; nextPen++)
					{
						int endPoint = (nextPen < (NumberOfPens - 1)) ? m_NextGraphStart[nextPen+1] : m_nNumberOfPoints;
						for (i = m_NextGraphStart[nextPen]; i < endPoint; i++)
						{
							m_Pens[nextPen].Width = 2;	// works in vertical direction only
							g.DrawLine(m_Pens[nextPen], ((PointI)m_points[i]).X, ((PointI)m_points[i]).Y,
								((PointI)m_points[i]).X + 1, ((PointI)m_points[i]).Y);
						}
					}
				}
			}
		}

		private int TranslateX(float x)
		{
			int tmp = 0;
			tmp = this.ClientRectangle.Left + kXAxisIndent + (int)((float)x*kScreenScale/XTickValue) -
				(int)(XOrigin*kScreenScale/XTickValue);
			return tmp;
		}
	
		private int TranslateY(float y)
		{
			int tmp = 0;
			tmp = this.ClientRectangle.Bottom - (((int)((float)y*kScreenScale/YTickValue) + kYAxisIndent -
				(int)(YOrigin*kScreenScale/YTickValue)));
			return tmp;
		}

		private float UntranslateX(int x)
		{
			float tmp = 0;
			tmp = (float)(x - this.ClientRectangle.Left + kXAxisIndent) * XTickValue + XOrigin;
			return tmp;
		}

		private float UntranslateY(int y)
		{
			float tmp = 0;
			tmp = ((float)(this.ClientRectangle.Bottom - (y + kYAxisIndent))) * YTickValue + YOrigin;
			return tmp;
		}

		public float XValue
		{
			get
			{
				return (float)(UntranslateX(((PointI)m_points[m_nCurrentIndex]).X) * m_fXTickValue + XOrigin);
			}
			set
			{
				PointI p = ((PointI)m_points[m_nCurrentIndex]);
				p.X = TranslateX(value);
			}
		}

		public float YValue
		{
			get
			{
				return (float)(UntranslateY(((PointI)m_points[m_nCurrentIndex]).Y) * m_fYTickValue + YOrigin);
			}
			set
			{
				PointI p = ((PointI)m_points[m_nCurrentIndex]);
				p.Y = TranslateY(value) ;
			}
		}

		public Point AbsolutePoint
		{
			get
			{
				return ((PointI)m_points[m_nCurrentIndex]).ToPoint();
			}
			set
			{
				m_points[m_nCurrentIndex] = new PointI(value);	
			}
		}

		public int CurrentIndex
		{
			get
			{
				return m_nCurrentIndex;
			}
			set
			{
				m_nCurrentIndex = value;	
			}
		}

		public float XOrigin
		{
			get
			{
				return m_ptXOrigin;
			}
			set
			{
				m_ptXOrigin = value;	
				Invalidate();
			}
		}

		public float YOrigin
		{
			get
			{
				return m_ptYOrigin;
			}
			set
			{
				m_ptYOrigin = value;	
				Invalidate();
			}
		}

		public int NumberOfPoints
		{
			get
			{
				return m_nNumberOfPoints;
			}
			set
			{
				m_nNumberOfPoints = value;
			}
		}

		public float XTickValue
		{
			get
			{
				return m_fXTickValue;	
			}
			set
			{
				m_fXTickValue = value;	
				Invalidate();
			}
		}
		
		public float YTickValue
		{
			get
			{
				return m_fYTickValue;	
			}
			set
			{
				m_fYTickValue = value;	
				Invalidate();
			}
		}

		public string LabelX
		{
			get
			{
				return m_LabelX;
			}
			set
			{
				m_LabelX = value;
			}
		}

		public string LabelY
		{
			get
			{
				return m_LabelY;
			}
			set
			{
				m_LabelY = value;
			}
		}

		public bool ConnectPointsFlag
		{
			get
			{
				return m_bConnectPoints;
			}
			set
			{
				m_bConnectPoints = value;
			}
		}

		public void AddPoint(float x, float y)
		{
			m_points.Add(new PointI());
			XValue = x;
			YValue = y;
			CurrentIndex ++;
			NumberOfPoints ++;
		}

		public void Reset() 
		{
			CurrentIndex = 0;
			NumberOfPoints = 0;
			m_points.Clear();
		}

		private void DrawXAxis(Graphics g)
		{
			Pen myPen = new Pen(Color.Blue, 1);
			Pen myAxisPen = new Pen(Color.DarkSlateGray, 1);
			SolidBrush myBrush = new SolidBrush(Color.Black);
			Font theAxisFont = new Font("Arial", 8);
			g.DrawString(m_LabelX, GraphFont, Brushes.Blue,
				new PointF(kXAxisIndent + 50, ClientRectangle.Bottom - 16), new StringFormat());
			float count = XOrigin;
			int tickcount = 0;
			for (float i = kXAxisIndent; i < this.ClientRectangle.Width; i+=kScreenScale)
			{
				int t = 3;
				if ((tickcount % 5) == 0)
				{
					float xcalcValue = count;
					string strCalcValue = xcalcValue.ToString("0.#");
					g.DrawString(strCalcValue, theAxisFont, myBrush, i,
						this.ClientRectangle.Bottom - (kYAxisIndent - theAxisFont.Height/3) + 3);
					t = 6;
				}
				g.DrawLine(ThePen, this.ClientRectangle.Left + i, this.ClientRectangle.Bottom - kYAxisIndent - t + 1,
					this.ClientRectangle.Left + i, this.ClientRectangle.Bottom - kYAxisIndent + t + 1);
				count += XTickValue;
				tickcount++;
			}
			g.DrawLine(ThePen, ClientRectangle.Left + kXAxisIndent, ClientRectangle.Bottom - kYAxisIndent + 1,
				ClientRectangle.Right, ClientRectangle.Bottom - kYAxisIndent + 1);
			myPen.Dispose();
			myBrush.Dispose();
			myAxisPen.Dispose();
			theAxisFont.Dispose();
		} // DrawXAxis

		private void DrawYAxis(Graphics g)
		{
			g.DrawLine(ThePen, ClientRectangle.Left + kXAxisIndent,
				ClientRectangle.Top, ClientRectangle.Left + kXAxisIndent, ClientRectangle.Bottom  - kYAxisIndent);
			StringFormat theFormat = new StringFormat(StringFormatFlags.DirectionVertical);
			Pen myPen = new Pen(Color.Blue, 1);
			Pen myAxisPen = new Pen(Color.DarkSlateGray, 1);
			SolidBrush myBrush = new SolidBrush(Color.SlateGray); 
			Font theAxisFont = new Font("Arial", 8);
			g.DrawString(m_LabelY, GraphFont, Brushes.Blue,
				new PointF(ClientRectangle.Left - 2 + kXAxisIndent/2, ClientRectangle.Top + 30), theFormat);
			float count = YOrigin;
			int tickcount = 0;
			for (float i = this.ClientRectangle.Height - kYAxisIndent; i >= -2; i -= kScreenScale)
			{
				int t = 3;
				if ((tickcount % 5) == 0)
				{
					float ycalcValue = count;
					string strCalcValue = string.Format("{0,6:0.#}", ycalcValue);
					g.DrawString(strCalcValue, theAxisFont, myBrush,
						this.ClientRectangle.Left + kXAxisIndent - theAxisFont.Height - 24, i);
					t = 6;
				}
				g.DrawLine(ThePen, this.ClientRectangle.Left + kXAxisIndent - t,
					this.ClientRectangle.Bottom - (i + kYAxisIndent), this.ClientRectangle.Left + kXAxisIndent + t,
					this.ClientRectangle.Bottom - (i + kYAxisIndent));
				count += YTickValue;
				tickcount++;
			}		
			myPen.Dispose();
			myBrush.Dispose();
			myAxisPen.Dispose();
			theAxisFont.Dispose();
		} // DrawYAxis

		public Color LineColor
		{
			get
			{
				return ThePen.Color;
			}
			set
			{
				ThePen.Dispose();
				ThePen = new Pen(value);
				Invalidate();
			}
		}

		public void SetPen(int num, Color aColor)
		{
			if ((num < 1) || (num > NumberOfPens))
			{
				return;
			}
			m_Pens[num - 1] = new Pen(aColor, 2);	// replace the pen
		}

		public void AddPen(int n, Color aColor, float width)
		{
			if (n >= 5) return;
			m_NextGraphStart[n] = this.m_nNumberOfPoints;
			m_Pens[n] = new Pen(aColor, width);
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>

		private void InitializeComponent()
		{
			// 
			// PlotControl
			// 
			this.Name = "PlotControl";
			this.Size = new System.Drawing.Size(128, 120);
			this.SizeChanged += new System.EventHandler(this.PlotControl_SizeChanged);

		}
		#endregion

		private void PlotControl_SizeChanged(object sender, System.EventArgs e)
		{
			Invalidate();
		}
		/// <summary>
		/// Summary description for PointI.
		/// </summary>
		public class PointI
		{
			public int X = 0;
			public int Y = 0;

			public PointI()
			{
			}

			public PointI(Point p)
			{
				X = p.X;
				Y = p.Y;
			}

			public PointI(int x, int y)
			{
				X = x;
				Y = y;
			}

			public Point ToPoint()
			{
				return new Point(X, Y);
			}
		}
	}
}
